import { createRequire } from 'node:module';
import * as os from 'node:os';
import * as path from 'node:path';
import logUpdate from 'log-update';
import sade from 'sade';
import tinydate from 'tinydate';
import Crawler from 'simplecrawler';
import globrex from 'globrex';
import { fileURLToPath } from 'node:url';
import { Worker } from 'node:worker_threads';
import * as childProcess from 'node:child_process';
import { getChromePath } from 'chrome-launcher';
import * as fs from 'node:fs/promises';

let FORCE_COLOR, NODE_DISABLE_COLORS, NO_COLOR, TERM, isTTY = true;
if (typeof process !== "undefined") {
  ({ FORCE_COLOR, NODE_DISABLE_COLORS, NO_COLOR, TERM } = process.env || {});
  isTTY = process.stdout && process.stdout.isTTY;
}
const $ = {
  enabled: !NODE_DISABLE_COLORS && NO_COLOR == null && TERM !== "dumb" && (FORCE_COLOR != null && FORCE_COLOR !== "0" || isTTY)
};
function init(x, y) {
  let rgx = new RegExp(`\\x1b\\[${y}m`, "g");
  let open = `\x1B[${x}m`, close = `\x1B[${y}m`;
  return function(txt) {
    if (!$.enabled || txt == null)
      return txt;
    return open + (!!~("" + txt).indexOf(close) ? txt.replace(rgx, close + open) : txt) + close;
  };
}
const dim = init(2, 22);
const underline = init(4, 24);
const red = init(31, 39);
const green = init(32, 39);
const yellow = init(33, 39);
const blue = init(34, 39);
const gray = init(90, 39);

const noop = () => {
};
const asyncIteratorQueue = () => {
  const queue = [];
  let isFinished = false;
  let unpauseGenerator = noop;
  const itemAddedListeners = [];
  return {
    push(...items) {
      for (const callback of itemAddedListeners) {
        for (const addedItem of items)
          callback(addedItem);
      }
      queue.push(...items);
      unpauseGenerator();
    },
    finish() {
      isFinished = true;
      unpauseGenerator();
    },
    onItemAdded(callback) {
      itemAddedListeners.push(callback);
    },
    async *[Symbol.asyncIterator]() {
      let i = 0;
      while (true) {
        while (i < queue.length) {
          yield queue[i];
          i++;
        }
        if (isFinished)
          return;
        await new Promise((resolve) => {
          unpauseGenerator = resolve;
        });
      }
    }
  };
};

const globToRegex = (glob) => globrex(glob.replace(/\/$/, ""), { globstar: true, extended: true }).regex;
const createUrlFilter = (includeGlob, excludeGlob) => {
  const pathIncludeRegexes = includeGlob.map(globToRegex);
  const pathExcludeRegexes = excludeGlob.map(globToRegex);
  return ({ path }) => {
    const withoutTrailingSlash = path.replace(/\/$/, "");
    return (pathIncludeRegexes.length === 0 || pathIncludeRegexes.some((regex) => regex.test(withoutTrailingSlash))) && !pathExcludeRegexes.some((regex) => regex.test(withoutTrailingSlash));
  };
};

function crawl(initialUrl, opts, console) {
  const crawler = new Crawler(initialUrl);
  if (opts.crawlerUserAgent)
    crawler.userAgent = opts.crawlerUserAgent;
  crawler.respectRobotsTxt = !opts.ignoreRobotsTxt;
  if (opts.maxCrawlDepth !== void 0)
    crawler.maxDepth = opts.maxCrawlDepth;
  const initialPath = new URL(initialUrl).pathname;
  const resultsQueue = asyncIteratorQueue();
  crawler.addFetchCondition(createUrlFilter(opts.includePathGlob.length > 0 ? [...opts.includePathGlob, initialPath] : [], opts.excludePathGlob));
  crawler.on("fetchcomplete", (queueItem, _responseBuffer, response) => {
    const url = queueItem.url;
    const contentType = response.headers["content-type"];
    if (!contentType || !/html/i.test(contentType))
      return;
    const statusCode = response.statusCode;
    if (!contentType || !statusCode)
      return;
    resultsQueue.push(url);
  });
  crawler.on("complete", () => {
    resultsQueue.finish();
  });
  const logWarning = (queueItem, response) => {
    console.warn(`${yellow("\u26A0")} Error fetching (${response.statusCode}): ${queueItem.url}`);
  };
  crawler.on("fetcherror", logWarning);
  crawler.on("fetch404", logWarning);
  crawler.on("fetch410", logWarning);
  crawler.start();
  return resultsQueue;
}

const createLighthouseRunner = (lighthouseRunOpts) => {
  const worker = new Worker(path.join(path.dirname(fileURLToPath(import.meta.url)), "lighthouse-worker.js"));
  const runner = {
    worker,
    isFree: true,
    freePromise: null,
    async run(url) {
      this.isFree = false;
      worker.postMessage({ type: "runLighthouse", url, lighthouseRunOpts });
      const lighthouseReportPromise = new Promise((resolve, reject) => {
        const workerListener = (message) => {
          if (!isLighthouseReport(message))
            return;
          resolve(message);
          worker.removeListener("message", workerListener);
          worker.removeListener("error", errorListener);
        };
        const errorListener = (error) => {
          reject(error);
          worker.removeListener("message", workerListener);
          worker.removeListener("error", errorListener);
        };
        worker.addListener("message", workerListener);
        worker.addListener("error", errorListener);
      });
      const newFreePromise = this.freePromise.then(() => lighthouseReportPromise).then(() => {
        if (this.freePromise === newFreePromise) {
          this.isFree = true;
        }
        return runner;
      });
      this.freePromise = newFreePromise;
      return lighthouseReportPromise;
    }
  };
  runner.freePromise = Promise.resolve(runner);
  return runner;
};
const isLighthouseReport = (report) => typeof report === "object" && report !== null && "categories" in report && "audits" in report && "lighthouseVersion" in report;
const initWorkerThreads = (opts) => {
  const lighthouseRunners = [];
  const getNextAvailable = async () => {
    for (const lighthouseRunner of lighthouseRunners)
      if (lighthouseRunner.isFree)
        return lighthouseRunner;
    if (lighthouseRunners.length < opts.lighthouseConcurrency) {
      const lighthouseRunner = createLighthouseRunner(opts.lighthouseRunOpts);
      lighthouseRunners.push(lighthouseRunner);
      return lighthouseRunner;
    }
    return Promise.race(lighthouseRunners.map((runner) => runner.freePromise));
  };
  const close = () => Promise.all(lighthouseRunners.map(async (lighthouseRunner) => {
    lighthouseRunner.worker.postMessage({ type: "close" });
    await new Promise((resolve) => {
      lighthouseRunner.worker.on("exit", () => resolve());
    });
  }));
  cleanupFunctions.push(close);
  return {
    getNextAvailable,
    close
  };
};
const cleanupFunctions = [];
const cleanup = (exitCode) => {
  const exit = () => process.exit(exitCode);
  Promise.all(cleanupFunctions.map((fn) => fn())).then(exit).catch(exit);
  setTimeout(exit, 200);
};
process.on("SIGINT", cleanup);
process.on("SIGTERM", cleanup);
process.on("uncaughtException", (error) => {
  console.error(error);
  cleanup(1);
});

const BYTE_UNITS = [
  "B",
  "kB",
  "MB",
  "GB",
  "TB",
  "PB",
  "EB",
  "ZB",
  "YB"
];
const BIBYTE_UNITS = [
  "B",
  "kiB",
  "MiB",
  "GiB",
  "TiB",
  "PiB",
  "EiB",
  "ZiB",
  "YiB"
];
const BIT_UNITS = [
  "b",
  "kbit",
  "Mbit",
  "Gbit",
  "Tbit",
  "Pbit",
  "Ebit",
  "Zbit",
  "Ybit"
];
const BIBIT_UNITS = [
  "b",
  "kibit",
  "Mibit",
  "Gibit",
  "Tibit",
  "Pibit",
  "Eibit",
  "Zibit",
  "Yibit"
];
const toLocaleString = (number, locale, options) => {
  let result = number;
  if (typeof locale === "string" || Array.isArray(locale)) {
    result = number.toLocaleString(locale, options);
  } else if (locale === true || options !== void 0) {
    result = number.toLocaleString(void 0, options);
  }
  return result;
};
function prettyBytes(number, options) {
  if (!Number.isFinite(number)) {
    throw new TypeError(`Expected a finite number, got ${typeof number}: ${number}`);
  }
  options = {
    bits: false,
    binary: false,
    ...options
  };
  const UNITS = options.bits ? options.binary ? BIBIT_UNITS : BIT_UNITS : options.binary ? BIBYTE_UNITS : BYTE_UNITS;
  if (options.signed && number === 0) {
    return ` 0 ${UNITS[0]}`;
  }
  const isNegative = number < 0;
  const prefix = isNegative ? "-" : options.signed ? "+" : "";
  if (isNegative) {
    number = -number;
  }
  let localeOptions;
  if (options.minimumFractionDigits !== void 0) {
    localeOptions = { minimumFractionDigits: options.minimumFractionDigits };
  }
  if (options.maximumFractionDigits !== void 0) {
    localeOptions = { maximumFractionDigits: options.maximumFractionDigits, ...localeOptions };
  }
  if (number < 1) {
    const numberString2 = toLocaleString(number, options.locale, localeOptions);
    return prefix + numberString2 + " " + UNITS[0];
  }
  const exponent = Math.min(Math.floor(options.binary ? Math.log(number) / Math.log(1024) : Math.log10(number) / 3), UNITS.length - 1);
  number /= (options.binary ? 1024 : 1e3) ** exponent;
  if (!localeOptions) {
    number = number.toPrecision(3);
  }
  const numberString = toLocaleString(Number(number), options.locale, localeOptions);
  const unit = UNITS[exponent];
  return prefix + numberString + " " + unit;
}

const getRunInfo = async (command, lighthouseParadeVersion, lighthouseVersion) => {
  const chromePath = getChromePath();
  return {
    command,
    time: new Date().toString(),
    versions: {
      npm: await runWithTimeoutAndGetStdout("npm --version"),
      chrome: await runWithTimeoutAndGetStdout(`${chromePath.replace(/\s/g, "\\$&")} --version`),
      lighthouse: lighthouseVersion,
      lighthouseParade: lighthouseParadeVersion,
      node: `${process.versions.node} (${process.arch})`
    },
    system: {
      operatingSystem: `${os.platform()} ${os.release()}`,
      memory: prettyBytes(os.totalmem()),
      cpus: `(${os.cpus().length}) ${os.cpus()[0].model}`
    }
  };
};
const runWithTimeoutAndGetStdout = (command) => new Promise((resolve) => {
  let resolved = false;
  childProcess.exec(command, (_error, stdout) => {
    resolved = true;
    resolve(stdout.trim() || "?");
  });
  setTimeout(() => {
    if (!resolved)
      resolve("?");
  }, 2e3);
});

const makeCSVRow = (cells) => `${cells.join(",")}
`;
const createCSVOutputWriter = async (filePath) => {
  const outputFile = await fs.open(filePath, "w");
  let mutexPromise = Promise.resolve();
  return {
    async writeHeader(columns) {
      mutexPromise = mutexPromise.then(async () => {
        await outputFile.write(makeCSVRow(["", ...columns.map((c) => c.lighthouseCategory)]) + makeCSVRow([
          "URL",
          ...columns.map((c) => c.name + (c.nameDetail ? ` (${c.nameDetail})` : ""))
        ]));
      });
      await mutexPromise;
    },
    async addEntry(url, rowValues) {
      mutexPromise = mutexPromise.then(async () => {
        await outputFile.write(makeCSVRow([url, ...rowValues]));
      });
      await mutexPromise;
    },
    async complete() {
      await outputFile.close();
    }
  };
};

const createGoogleSheetsOutputWriter = async (...args) => {
  const mod = await import('./index-bbab2464.js');
  return mod.createGoogleSheetsOutputWriter(...args);
};
var ColumnType = /* @__PURE__ */ ((ColumnType2) => {
  ColumnType2[ColumnType2["CategoryScore"] = 0] = "CategoryScore";
  ColumnType2[ColumnType2["AuditScore"] = 1] = "AuditScore";
  ColumnType2[ColumnType2["AuditBoolean"] = 2] = "AuditBoolean";
  ColumnType2[ColumnType2["AuditValue"] = 3] = "AuditValue";
  return ColumnType2;
})(ColumnType || {});
const adaptLHRToOutputWriter = (outputWriter, command, lighthouseParadeVersion) => {
  let mutexPromise = Promise.resolve();
  let hasWrittenHeader = false;
  const columns = [];
  return {
    complete: () => outputWriter.complete(),
    async addEntry(report) {
      if (!hasWrittenHeader) {
        mutexPromise = mutexPromise.then(async () => {
          hasWrittenHeader = true;
          for (const category of Object.values(report.categories)) {
            columns.push({
              name: category.title,
              nameDetail: "category score",
              lighthouseCategory: category.title,
              field: {
                type: 0 /* CategoryScore */,
                category: category.id
              }
            });
          }
          for (const category of Object.values(report.categories)) {
            for (const audit of Object.values(category.auditRefs)) {
              const auditData = report.audits[audit.id];
              if (auditData.scoreDisplayMode === "numeric") {
                columns.push({
                  name: auditData.title,
                  nameDetail: "score",
                  lighthouseCategory: category.title,
                  field: {
                    type: 1 /* AuditScore */,
                    hasAuditValueColumn: auditData.numericValue !== void 0,
                    audit: audit.id
                  }
                });
              }
              if (auditData.scoreDisplayMode === "binary") {
                columns.push({
                  name: auditData.title,
                  lighthouseCategory: category.title,
                  field: {
                    type: 2 /* AuditBoolean */,
                    audit: audit.id
                  }
                });
              }
              if (auditData.numericValue) {
                columns.push({
                  name: auditData.title,
                  nameDetail: auditData.numericUnit,
                  lighthouseCategory: category.title,
                  field: {
                    type: 3 /* AuditValue */,
                    unit: auditData.numericUnit || "",
                    audit: audit.id
                  }
                });
              }
            }
          }
          await Promise.all([
            outputWriter.writeHeader(columns),
            outputWriter.writeRunInfo && getRunInfo(command, lighthouseParadeVersion, report.lighthouseVersion).then((runInfo) => {
              var _a;
              return (_a = outputWriter.writeRunInfo) == null ? void 0 : _a.call(outputWriter, runInfo);
            })
          ]);
        });
        await mutexPromise;
      }
      const rowValues = columns.map((c) => {
        if (c.field.type === 2 /* AuditBoolean */) {
          const val2 = report.audits[c.field.audit].score;
          if (typeof val2 === "number")
            return val2 ? "PASS" : "FAIL";
          return "";
        }
        const val = c.field.type === 1 /* AuditScore */ ? report.audits[c.field.audit].score : c.field.type === 3 /* AuditValue */ ? report.audits[c.field.audit].numericValue : report.categories[c.field.category].score;
        return typeof val === "number" ? String(c.field.type === 3 /* AuditValue */ ? val : val * 100) : "";
      });
      return outputWriter.addEntry(report.finalUrl, rowValues);
    }
  };
};
const combineOutputWriters = (outputWriters) => {
  let mutexPromise = Promise.resolve();
  return {
    async writeHeader(...args) {
      mutexPromise = mutexPromise.then(() => Promise.all(outputWriters.map((writer) => writer.writeHeader(...args))));
      await mutexPromise;
    },
    async writeRunInfo(...args) {
      mutexPromise = mutexPromise.then(() => Promise.all(outputWriters.map((writer) => {
        var _a;
        return (_a = writer.writeRunInfo) == null ? void 0 : _a.call(writer, ...args);
      })));
      await mutexPromise;
    },
    async addEntry(...args) {
      mutexPromise = mutexPromise.then(() => Promise.all(outputWriters.map((writer) => writer.addEntry(...args))));
      await mutexPromise;
    },
    async complete() {
      mutexPromise = mutexPromise.then(() => Promise.all(outputWriters.map((writer) => writer.complete())));
      await mutexPromise;
    }
  };
};

const getOutputWriter = async (outputs, initialUrl, command, lighthouseParadeVersion) => {
  const outputWriters = await Promise.all(outputs.map((output) => {
    if (output === "google-sheets") {
      return createGoogleSheetsOutputWriter(initialUrl);
    }
    const googleSheetsPrefix = "google-sheets:";
    if (output.startsWith(googleSheetsPrefix)) {
      return createGoogleSheetsOutputWriter(initialUrl, output.slice(googleSheetsPrefix.length));
    }
    const ext = path.extname(output);
    if (ext === ".csv")
      return createCSVOutputWriter(output);
    throw new Error(`Invalid output format: ${ext} (${output}). Expected <filename>.csv, or google-sheets, or google-sheets:"<title>"`);
  }));
  const outputWriter = combineOutputWriters(outputWriters);
  return adaptLHRToOutputWriter(outputWriter, command, lighthouseParadeVersion);
};
var State = /* @__PURE__ */ ((State2) => {
  State2[State2["Pending"] = 0] = "Pending";
  State2[State2["InProgress"] = 1] = "InProgress";
  State2[State2["Success"] = 2] = "Success";
  State2[State2["Failure"] = 3] = "Failure";
  return State2;
})(State || {});
const main = async (initialUrl, opts, console, command, lighthouseParadeVersion) => {
  const state = /* @__PURE__ */ new Map();
  const outputWriter = await getOutputWriter(opts.outputs, initialUrl, command, lighthouseParadeVersion);
  const start = async () => {
    const lighthouseRunners = initWorkerThreads(opts);
    const lighthousePromises = [];
    const crawlIterator = crawl(initialUrl, opts, console);
    crawlIterator.onItemAdded((url) => {
      state.set(url, { state: 0 /* Pending */ });
    });
    for await (const url of crawlIterator) {
      const lighthouseRunner = await lighthouseRunners.getNextAvailable();
      state.set(url, { state: 1 /* InProgress */ });
      lighthousePromises.push(lighthouseRunner.run(url).then(async (lighthouseReport) => {
        await outputWriter.addEntry(lighthouseReport);
        state.set(url, { state: 2 /* Success */ });
      }).catch((error) => {
        state.set(url, { state: 3 /* Failure */, error });
      }));
    }
    await Promise.all(lighthousePromises);
    await outputWriter.complete();
    await lighthouseRunners.close();
  };
  return {
    state,
    start
  };
};

const require = createRequire(import.meta.url);
const { version } = require("../package.json");
const symbols = {
  error: red("\u2716"),
  success: green("\u2714")
};
const toArray = (input) => Array.isArray(input) ? input : input === void 0 ? [] : [input];
const isFullURL = (path2) => {
  try {
    new URL(path2);
    return true;
  } catch {
  }
  return false;
};
const allLighthouseCategories = [
  "accessibility",
  "best-practices",
  "performance",
  "pwa",
  "seo"
];
sade("lighthouse-parade <url>", true).version(version).example('https://cloudfour.com --exclude-path-glob "/thinks/*" --max-crawl-depth 2 --output cloudfour-a.csv').describe("Crawls the site at the provided URL, recording the lighthouse scores for each URL found.").option("--output, -o", 'The output file(s). Can be specified multiple times, e.g. -o cloudfour-a.csv -o google-sheets -o google-sheets:"Spreadsheet Name"').option("--ignore-robots-txt", "Crawl pages even if they are listed in the site's robots.txt", false).option("--crawler-user-agent", "Pass a user agent string to be used by the crawler (not by Lighthouse)").option("--lighthouse-concurrency", "Control the maximum number of ligthhouse reports to run concurrently", os.cpus().length - 1).option("--lh:only-categories", `Only run the specified lighthouse categories. Available categories: ${allLighthouseCategories.join(", ")}. Multiple can be specified using commas, e.g. --lh:only-categories=accessibility,seo. If not specified, all categories will be used.`).option("--max-crawl-depth", "Control the maximum depth of crawled links. 1 means only the entry page will be used. 2 means the entry page and any page linked directly from the entry page will be used.").option("--include-path-glob", "Specify a glob (in quotes) for paths to match. Links to non-matched paths will not be crawled. The entry page will be crawled regardless of this flag. This flag can be specified multiple times to allow multiple paths. `*` matches one url segment, `**` matches multiple segments. Trailing slashes are ignored.").option("--exclude-path-glob", "Specify a glob (in quotes) for paths to exclude. Links to matched paths will not be crawled. The entry page will be crawled regardless of this flag. This flag can be specified multiple times to exclude multiple paths. `*` matches one url segment, `**` matches multiple segments. Trailing slashes are ignored.").action(async (url, opts) => {
  if (!isFullURL(url)) {
    throw new Error("The input URL must be a full URL (with protocol, etc.)");
  }
  const ignoreRobotsTxt = opts["ignore-robots-txt"];
  const outputs = toArray(opts.output);
  if (outputs.length === 0) {
    const siteName = new URL(url).hostname.replace(/[^\da-z]+/gi, "-");
    const date = tinydate("{YYYY}-{MM}-{DD}-{HH}:{mm}")(new Date());
    outputs.push(`lighthouse-${siteName}-${date}.csv`);
  }
  const crawlerUserAgent = opts["crawler-user-agent"];
  if (crawlerUserAgent !== void 0 && typeof crawlerUserAgent !== "string") {
    throw new Error("--crawler-user-agent must be a string");
  }
  const maxCrawlDepth = opts["max-crawl-depth"];
  if (maxCrawlDepth !== void 0 && typeof maxCrawlDepth !== "number") {
    throw new Error("--max-crawl-depth must be a number");
  }
  const includePathGlob = toArray(opts["include-path-glob"]);
  if (includePathGlob.some((glob) => typeof glob !== "string")) {
    throw new Error("--include-path-glob must be string(s)");
  }
  if (includePathGlob.some(isFullURL)) {
    throw new Error("--include-path-glob must be path(s), not full URL(s)");
  }
  const excludePathGlob = toArray(opts["exclude-path-glob"]);
  if (excludePathGlob.some((glob) => typeof glob !== "string")) {
    throw new Error("--exclude-path-glob must be string(s)");
  }
  if (excludePathGlob.some(isFullURL)) {
    throw new Error("--exclude-path-glob must be path(s), not full URL(s)");
  }
  const lighthouseConcurrency = opts["lighthouse-concurrency"];
  if (typeof lighthouseConcurrency !== "number") {
    throw new TypeError("--lighthouse-concurrency must be a number");
  }
  const lighthouseCategories = toArray(opts["lh:only-categories"]).flatMap((c) => String(c).split(","));
  for (const category of lighthouseCategories) {
    if (!allLighthouseCategories.includes(category)) {
      throw new Error(`Invalid lighthouse category: ${category}. --lh:only-categories must be a combination of: ${allLighthouseCategories.join(", ")}`);
    }
  }
  if (lighthouseCategories.length === 0) {
    lighthouseCategories.push(...allLighthouseCategories);
  }
  const frames = ["\u280B", "\u2819", "\u2839", "\u2838", "\u283C", "\u2834", "\u2826", "\u2827", "\u2807", "\u280F"].map((f) => blue(f));
  let i = 0;
  const printLine = (url2, urlState) => {
    const frame = frames[i];
    const statusIcon = urlState.state === State.Failure ? symbols.error : urlState.state === State.Pending ? " " : urlState.state === State.InProgress ? frame : symbols.success;
    let output = `${statusIcon} ${url2}`;
    if (urlState.state === State.Failure) {
      output += `
  ${gray(urlState.error.toString())}`;
    }
    return output;
  };
  const completedURLs = /* @__PURE__ */ new Set();
  const render = (urlStates) => {
    logUpdate.clear();
    const pendingUrls = [];
    const currentUrls = [];
    for (const [url2, urlState] of urlStates.entries()) {
      if (urlState.state === State.Success || urlState.state === State.Failure) {
        if (!completedURLs.has(urlState)) {
          completedURLs.add(urlState);
          console.log(printLine(url2, urlState));
        }
        continue;
      }
      const line = `${printLine(url2, urlState)}
`;
      if (urlState.state === State.Pending)
        pendingUrls.push(line);
      else
        currentUrls.push(line);
    }
    const numPendingToDisplay = Math.min(Math.max(process.stdout.rows - currentUrls.length - 3, 1), pendingUrls.length);
    const numHiddenUrls = numPendingToDisplay === pendingUrls.length ? "" : dim(`
...And ${pendingUrls.length - numPendingToDisplay} more pending`);
    logUpdate(currentUrls.join("") + pendingUrls.slice(0, numPendingToDisplay).join("") + numHiddenUrls);
  };
  const printAboveLogUpdate = (cb) => {
    logUpdate.clear();
    cb();
    render(runStatus.state);
  };
  const modifiedConsole = {
    log: (...messages) => printAboveLogUpdate(() => console.log(...messages)),
    warn: (...messages) => printAboveLogUpdate(() => console.warn(...messages)),
    error: (...messages) => printAboveLogUpdate(() => console.error(...messages))
  };
  const command = [
    path.basename(process.argv[1]),
    ...process.argv.slice(2).map((chunk) => chunk.includes("*") ? JSON.stringify(chunk) : chunk)
  ].join(" ");
  const lighthouseRunOpts = {
    categories: lighthouseCategories
  };
  const runStatus = await main(url, {
    includePathGlob,
    excludePathGlob,
    ignoreRobotsTxt,
    outputs,
    maxCrawlDepth,
    crawlerUserAgent,
    lighthouseConcurrency,
    lighthouseRunOpts
  }, modifiedConsole, command, version);
  const intervalId = setInterval(() => {
    i = (i + 1) % frames.length;
    render(runStatus.state);
  }, 80);
  await runStatus.start();
  render(runStatus.state);
  clearInterval(intervalId);
}).parse(process.argv);

export { ColumnType as C, blue as b, underline as u };
